ArrayList<Vertex> vertices;
ArrayList<Figur> faces;
ArrayList<Normal> normals;

// Skalierungsfaktor für Geometrie
float scalef=800;
// Feld für Worte beim Dateilesen
String [] token;

float dirLightX = 0.5;
float dirLightY = 0.5;
  
float geoRotX = PI/2;
float geoRotZ = PI;

//////////////////////////////////////////////////

class Vertex {
 float x,y,z;
 
 Vertex() { x=y=z=0; }
 Vertex(float px,float py,float pz) { x=px;y=py;z=pz; }
 Vertex(Vertex p) { x=p.x;y=p.y;z=p.z; }

 String toString() {
   String s = x + " " + y + " " + z;
   return s;
 }
}

//////////////////////////////////////////////////

class Normal {
 float x,y,z;
 
 Normal() { x=y=z=0; }
 Normal(float px,float py,float pz) {  x=px;y=py;z=pz; }
 Normal(Normal p) { x=p.x;y=p.y;z=p.z; }

 String toString() {
   String s = x + " " + y + " " + z;
   return s;
 }
}

///////////////////////////////////////////////////////////////

class Figur {
  ArrayList<Integer> p;
  ArrayList<Integer> t;
  ArrayList<Integer> n;
 
  Figur() { 
    p = new ArrayList<Integer>(3); 
    t = new ArrayList<Integer>(3); 
    n = new ArrayList<Integer>(3); 
  }
  Figur(int np) { 
    p = new ArrayList<Integer>(np); 
    t = new ArrayList<Integer>(np); 
    n = new ArrayList<Integer>(np); 
  }
  Figur(Figur f) { 
    p = new ArrayList<Integer>(f.p.size()); 
    t = new ArrayList<Integer>(f.t.size()); 
    n = new ArrayList<Integer>(f.n.size()); 
    for (int i=0;i<f.p.size();i++) p.add(f.p.get(i));
    for (int i=0;i<f.t.size();i++) t.add(f.t.get(i));
    for (int i=0;i<f.n.size();i++) n.add(f.n.get(i));
  }
  
  void addVertex(int pp) { p.add(new Integer(pp)); }
  void addTextureCoord(int pp) { t.add(new Integer(pp)); }
  void addNormal(int pp) { n.add(new Integer(pp)); }

  int getVertex(int i) { return p.get(i); } 
  int getTextureCoord(int i) { return t.get(i); }
 // int getNormal(int i) { return n.get(i); }
}

/////////////////////////////////////////////////////////////////////
//
// Geometrie liegt im Bereich [-0.5..0.5] nach der Normalisierung
//
/////////////////////////////////////////////////////////////////////

void normalizeGeometry() {

  Vertex p = (Vertex)vertices.get(1);
  
  float xmin=p.x,xmax=p.x;
  float ymin=p.y,ymax=p.y;
  float zmin=p.z,zmax=p.z;
 
  for (int i=1;i<vertices.size();i++) {
     p = (Vertex)vertices.get(i);
     xmin = min(xmin,p.x); xmax=max(xmax,p.x);
     ymin = min(ymin,p.y); ymax=max(ymax,p.y);
     zmin = min(zmin,p.z); zmax=max(zmax,p.z);
  }
  float scale = 1.0/max(xmax-xmin,max(ymax-ymin,zmax-zmin));

  for (int i=1;i<vertices.size();i++) {
     p = (Vertex)vertices.get(i);
     p.x = (-0.5*(xmax-xmin)+(p.x-xmin))*scale;
     p.y = (-0.5*(ymax-ymin)+(p.y-ymin))*scale;
     p.z = (-0.5*(zmax-zmin)+(p.z-zmin))*scale;
  }
}

///////////////////////////////////////////////////////////////
//
// Lese eine Zeile aus der Datei
//
///////////////////////////////////////////////////////////////

boolean readLine(BufferedReader input) {
  String line;
  
  // versuche eine Zeile aus der Datei zu lesen
  try {
    line = input.readLine();
  } catch (IOException e) { line = null; }
  
  // bei Fehler gebe false zurück (und springe raus
  if (line == null) return false;
 
  // wenn eine Kommentarzeile oder leere Zeile, lese die nächste Zeile
  if (((line.length()>0) && (line.charAt(0)=='#')) || (line.length()==0)) 
      return readLine(input);

  token = split(trim(line)," ");
  return true;
}

////////////////////////////////////////////////////////////////
//
// Lade Geometrie aus einer Datei
//
////////////////////////////////////////////////////////////////

void readGeometry(String loadPath) {

  BufferedReader reader = createReader(loadPath); 
  if (reader==null) {
     println("Cannot load file " + loadPath);
     exit();
  }
  
  vertices.add(new Vertex(0,0,0)); // dummy-Eintrag, da Liste mit Index 1 startet
  normals.add(new Normal(0,0,0));  // dummy-Eintrag
  while (readLine(reader)) {
    // ein Vertex
    if (token[0].equals("v")) 
       vertices.add(new Vertex(Float.valueOf(token[1]),
            Float.valueOf(token[2]),Float.valueOf(token[3])));
    // eine Normale
    if (token[0].equals("vn")) 
       normals.add(new Normal(Float.valueOf(token[1]),
            Float.valueOf(token[2]),Float.valueOf(token[3])));
    // eine Fläche
    if (token[0].equals("f")) {
       Figur f = new Figur(token.length-1);
       for (int i=1;i<token.length;i++) {
         String[] tt = split(trim(token[i]),"/");
         if ((tt.length>0) && (tt[0].length()>0)) 
              f.addVertex(Integer.valueOf(tt[0]));
         if ((tt.length>1) && (tt[1].length()>0)) 
              f.addTextureCoord(Integer.valueOf(tt[1]));
         if ((tt.length>2) && (tt[2].length()>0)) 
              f.addNormal(Integer.valueOf(tt[2]));
       }
       faces.add(f);
    }
  } 
  
  normalizeGeometry();
}
  
/////////////////////////////////////////////////////////////////////

void drawGeometry(float size) {
  scale(size,size,size);
  beginShape(TRIANGLES);
    for (int i=0;i<faces.size();i++) {
      Figur f = (Figur)faces.get(i);
      Vertex p1 = (Vertex)vertices.get(f.getVertex(0));
      Vertex p2 = (Vertex)vertices.get(f.getVertex(1));
      Vertex p3 = (Vertex)vertices.get(f.getVertex(2));
     // Normal n1 = (Normal)normals.get(f.getNormal(0));
    //  Normal n2 = (Normal)normals.get(f.getNormal(1));
     // Normal n3 = (Normal)normals.get(f.getNormal(2));
     // normal(n1.x,n1.y,n1.z);//deaktivieren wenn ohne Normalen...
      vertex(p1.x,p1.y,p1.z);
     // normal(n2.x,n2.y,n2.z);//deaktivieren wenn ohne Normalen...
      vertex(p2.x,p2.y,p2.z);
     // normal(n3.x,n3.y,n3.z);//deaktivieren wenn ohne Normalen...
      vertex(p3.x,p3.y,p3.z);
   }                         //gezeichnet werden soll.
  endShape();
}

/////////////////////////////////////////////////////////////////////

void setup() {
  size(1000,1000,P3D);  
  colorMode(RGB,255,255,255);
  noStroke();
      
  vertices = new ArrayList<Vertex>(10000);
  faces = new ArrayList<Figur>(10000);
  normals = new ArrayList<Normal>(10000);
  readGeometry("DerDenker.obj");
  //readGeometry("Medusa.obj");
  //readGeometry("Katze.obj");
  //readGeometry("data/DerDenker.obj");
  //readGeometry("DerDenker.obj");
  //readGeometry("Laurana50k.obj");
  //readGeometry("BMW850.obj");
  //readGeometry("utahteapot.obj");
 
}

///////////////////////////////////////////////////////////////////

void draw()  { 
  background(255);
 // stroke(0); //aktiviert man diese beiden Zeilen
 // strokeWeight(0.0005);//kann man die Dreieck-Shapes sehen
  if (mousePressed) {
    if (mouseButton==RIGHT) {
      dirLightY = map(mouseY,0,height,1,-1);
      dirLightX = map(mouseX,0,width,1,-1);
    } else {
      geoRotX = map(mouseY,0,height,0,-TWO_PI);//Rotation um die x-Achse
      geoRotZ = map(mouseX,0,width,0,-TWO_PI); //Rotation um die z-Achse
    }
  }
  ambientLight(100,100,100);
  // Beleuchte mit Lichtquelle je nach rechter Mausposition
  directionalLight(150, 150, 150, dirLightX, dirLightY, -1); 

  // verschiebe Objekt in Bildmitte
  translate(width/2, height/2, -30); 
  // rotiere um x-Achse je nach Höhe der Maus (linker Button)
  rotateX(geoRotX); 
  // rotiere um y-Achse je nach seitlicher Koordinate der Maus
  rotateZ(geoRotZ);   
  
  drawGeometry(scalef); 
} 

///////////////////////////////////////////////////////////////////

void keyPressed() {
  if (key=='+') scalef = min(10000,scalef*1.2);
  if (key=='-') scalef = max(100,scalef/1.2);
}
